@@ -1,7 +1,9 @@
-spLM <- function(formula, data = parent.frame(), coords, knots,
-                 starting, sp.tuning, priors, cov.model, 
-                 modified.pp = TRUE, n.samples, sub.samples, verbose=TRUE, n.report=100, ...){
-
+spLM <- function(formula, data = parent.frame(), coords, knots, 
+                 starting, tuning, priors, cov.model,
+                 modified.pp=TRUE, amcmc, n.samples,
+                 ##get.beta=FALSE, start=1, end, thin=1,
+                 verbose=TRUE, n.report=100, ...){
+  
   ####################################################
   ##Check for unused args
   ####################################################
@@ -13,7 +15,7 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
   }
 
   ####################################################
-  ##formula
+  ##Formula
   ####################################################
   if(missing(formula)){stop("error: formula must be specified")}
   
@@ -37,29 +39,74 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
   storage.mode(p) <- "integer"
   storage.mode(n) <- "integer"
 
+  ####################################################
+  ##sampling method
+  ####################################################
+  n.batch <- 0
+  batch.length <- 0
+  accept.rate <- 0
+  is.amcmc <- TRUE
+  
+  if(missing(amcmc)){
+   
+    if(missing(n.samples)){stop("error: n.samples needs to be specified")}
+    
+    n.batch <- n.samples
+    batch.length <- 1
+    is.amcmc <- FALSE
+        
+  }else{
+    
+    names(amcmc) <- tolower(names(amcmc))
+    
+    if(!"n.batch" %in% names(amcmc)){stop("error: n.batch must be specified in amcmc list")}
+    n.batch <- amcmc[["n.batch"]]
+    
+    if(!"batch.length" %in% names(amcmc)){stop("error: batch.length must be specified in amcmc list")}
+    batch.length <- amcmc[["batch.length"]]
+
+    if(!"accept.rate" %in% names(amcmc)){
+      warning("accept.rate was not specified in the amcmc list and was therefore set to the default 0.43")
+      accept.rate <- 0.43
+    }else{
+      accept.rate <- amcmc[["accept.rate"]]
+    }
+
+  }
+
+  storage.mode(is.amcmc) <- "integer"
+  storage.mode(n.batch) <- "integer"
+  storage.mode(batch.length) <- "integer"
+  storage.mode(accept.rate) <- "double"
+
+  ####################################################
+  ##Fit non-spatial model if specified
+  ####################################################
+  if(missing(coords)){
+    return(bayesLMRef(lm(Y~X-1), n.batch*batch.length))
+  }
+  
   ####################################################
   ##Distance matrices
   ####################################################
   
   ####################
   ##Coords
-  ####################
-  if(missing(coords)){stop("error: coords must be specified")}
+  #################### 
   if(!is.matrix(coords)){stop("error: coords must n-by-2 matrix of xy-coordinate locations")}
   if(ncol(coords) != 2 || nrow(coords) != n){
     stop("error: either the coords have more than two columns or then number of rows is different than
           data used in the model formula")
   }
   
-  coords.D <- as.matrix(dist(coords))
+  coords.D <- iDist(coords)
   storage.mode(coords.D) <- "double"
-  
+
   ####################
   ##Knots
   ####################
-
   is.pp <- FALSE
-
+  
   if(!missing(knots)){
     
     if(is.vector(knots) && length(knots) %in% c(2,3)){
@@ -113,92 +160,51 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
       stop("error: knots is misspecified")
     }
   }
-
+ 
   m <- 0
   knots.D <- 0
-  coords.knots.D <- 0
+  knots.coords.D <- 0
   
   if(is.pp){
-    knots.D <- as.matrix(dist(knot.coords))
+    knots.D <- iDist(knot.coords)
     m <- nrow(knots.D)
-    coords.knots.D <- matrix(0, n, m) ##this is for c^t
-
-    for(i in 1:n){
-      coords.knots.D[i,] <- sqrt((knot.coords[,1]-coords[i,1])^2+
-                                 (knot.coords[,2]-coords[i,2])^2)
-    }
-    
-    storage.mode(modified.pp) <- "integer"
-    storage.mode(m) <- "integer"
-    storage.mode(knots.D) <- "double"
-    storage.mode(coords.knots.D) <- "double"
+    knots.coords.D <- iDist(knot.coords, coords)
   }
 
+  storage.mode(modified.pp) <- "integer"
+  storage.mode(m) <- "integer"
+  storage.mode(knots.D) <- "double"
+  storage.mode(knots.coords.D) <- "double"
+  
   ####################################################
   ##Covariance model
   ####################################################
   if(missing(cov.model)){stop("error: cov.model must be specified")}
   if(!cov.model%in%c("gaussian","exponential","matern","spherical"))
     {stop("error: specified cov.model '",cov.model,"' is not a valid option; choose, from gaussian, exponential, matern, spherical.")}
-
-  ####################################################
-  ##Starting values
-  ####################################################
-
-  beta.starting <- 0
-  sigma.sq.starting <- 0
-  tau.sq.starting <- 0
-  phi.starting <- 0
-  nu.starting <- 0
-
-  nugget <- FALSE
-  
-  if(missing(starting)){stop("error: starting value list for the parameters must be specified")}
-  
-  names(starting) <- tolower(names(starting))   
-
-  if(!"beta" %in% names(starting)){
-    beta.starting <- coefficients(lm(Y~X-1))
-  }else{
-    beta.starting <- starting[["beta"]]
-  }
-  
-  if(!"sigma.sq" %in% names(starting)){stop("error: sigma.sq must be specified in starting value list")}
-  sigma.sq.starting <- starting[["sigma.sq"]][1]
-  
-  if("tau.sq" %in% names(starting)){
-    if(!"tau.sq" %in% names(starting)){stop("error: tau.sq must be specified in starting value list")}
-    tau.sq.starting <- starting[["tau.sq"]][1]
-    nugget <- TRUE
-  }
-  
-  if(!"phi" %in% names(starting)){stop("error: phi must be specified in starting value list")}
-  phi.starting <- starting[["phi"]][1]
-  
-  if(cov.model == "matern"){
-    if(!"nu" %in% names(starting)){stop("error: nu must be specified in starting value list")}
-    nu.starting <- starting[["nu"]][1]
-  }
-    
-  storage.mode(nugget) <- "integer"
-  storage.mode(beta.starting) <- "double"
-  storage.mode(phi.starting) <- "double"
-  storage.mode(sigma.sq.starting) <- "double"
-  storage.mode(tau.sq.starting) <- "double"
-  storage.mode(nu.starting) <- "double"
-
   
   ####################################################
   ##Priors
   ####################################################
+  beta.Norm <- 0
+  beta.prior <- "flat"
   sigma.sq.IG <- 0
   tau.sq.IG <- 0
   nu.Unif <- 0
   phi.Unif <- 0
+  nugget <- FALSE
 
   if(missing(priors)) {stop("error: prior list for the parameters must be specified")}
     
   names(priors) <- tolower(names(priors))
+
+  if("beta.norm" %in% names(priors)){
+    beta.Norm <- priors[["beta.norm"]]
+    if(!is.list(beta.Norm) || length(beta.Norm) != 2){stop("error: beta.Norm must be a list of length 2")}
+    if(length(beta.Norm[[1]]) != p ){stop(paste("error: beta.Norm[[1]] must be a vector of length, ",p, "",sep=""))}
+    if(length(beta.Norm[[2]]) != p^2 ){stop(paste("error: beta.Norm[[2]] must be a ",p,"x",p," covariance matrix",sep=""))}
+    beta.prior <- "normal"
+  }
   
   if(!"sigma.sq.ig" %in% names(priors)){stop("error: sigma.sq.IG must be specified")}
   sigma.sq.IG <- priors[["sigma.sq.ig"]]
@@ -206,12 +212,12 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
   if(!is.vector(sigma.sq.IG) || length(sigma.sq.IG) != 2){stop("error: sigma.sq.IG must be a vector of length 2")}
   if(any(sigma.sq.IG <= 0)){stop("error: sigma.sq.IG must be a positive vector of length 2")}
   
-  if(nugget){
-    if(!"tau.sq.ig" %in% names(priors)){stop("error: tau.sq.IG must be specified")}
+  if("tau.sq.ig" %in% names(priors)){
     tau.sq.IG <- priors[["tau.sq.ig"]]
     
     if(!is.vector(tau.sq.IG) || length(tau.sq.IG) != 2){stop("error: tau.sq.IG must be a vector of length 2")}
     if(any(tau.sq.IG <= 0)){stop("error: tau.sq.IG must be a positive vector of length 2")}
+    nugget <- TRUE
   }
   
   if(!"phi.unif" %in% names(priors)){stop("error: phi.Unif must be specified")}
@@ -233,34 +239,77 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
   storage.mode(tau.sq.IG) <- "double"
   storage.mode(nu.Unif) <- "double"
   storage.mode(phi.Unif) <- "double"
+  storage.mode(nugget) <- "integer"
+
+  ####################################################
+  ##Starting values
+  ####################################################
+  beta.starting <- 0
+  sigma.sq.starting <- 0
+  tau.sq.starting <- 0
+  phi.starting <- 0
+  nu.starting <- 0
+
+  if(missing(starting)){stop("error: starting value list for the parameters must be specified")}
+  
+  names(starting) <- tolower(names(starting))   
+  
+  if(is.pp){
+    if(!"beta" %in% names(starting)){
+      beta.starting <- as.vector(coefficients(lm(Y~X-1)))
+    }else{
+      beta.starting <- starting[["beta"]]
+    }
+  }
+ 
+  if(!"sigma.sq" %in% names(starting)){stop("error: sigma.sq must be specified in starting value list")}
+  sigma.sq.starting <- starting[["sigma.sq"]][1]
+
+  if(nugget){
+    if(!"tau.sq" %in% names(starting)){stop("error: a prior was spcified for tau.sq therefore tau.sq must be specified in starting value list")}
+    tau.sq.starting <- starting[["tau.sq"]][1]
+  }
+  
+  if(!"phi" %in% names(starting)){stop("error: phi must be specified in starting value list")}
+  phi.starting <- starting[["phi"]][1]
+  
+  if(cov.model == "matern"){
+    if(!"nu" %in% names(starting)){stop("error: nu must be specified in starting value list")}
+    nu.starting <- starting[["nu"]][1]
+  }
+
+  storage.mode(beta.starting) <- "double"
+  storage.mode(phi.starting) <- "double"
+  storage.mode(sigma.sq.starting) <- "double"
+  storage.mode(tau.sq.starting) <- "double"
+  storage.mode(nu.starting) <- "double"
 
   ####################################################
   ##Tuning values
   ####################################################
-
   phi.tuning <- 0
   sigma.sq.tuning <- 0
   tau.sq.tuning <- 0
   nu.tuning <- 0
   
-  if(missing(sp.tuning)){stop("error: sp.tuning value vector for the spatial parameters must be specified")}
+  if(missing(tuning)){stop("error: tuning value vector for the spatial parameters must be specified")}
   
-  names(sp.tuning) <- tolower(names(sp.tuning))
+  names(tuning) <- tolower(names(tuning))
   
-  if(!"sigma.sq" %in% names(sp.tuning)){stop("error: sigma.sq must be specified in tuning value list")}
-  sigma.sq.tuning <- sp.tuning[["sigma.sq"]][1]
+  if(!"sigma.sq" %in% names(tuning)){stop("error: sigma.sq must be specified in tuning value list")}
+  sigma.sq.tuning <- tuning[["sigma.sq"]][1]
   
   if(nugget){
-    if(!"tau.sq" %in% names(sp.tuning)){stop("error: tau.sq must be specified in tuning value list")}
-    tau.sq.tuning <- sp.tuning[["tau.sq"]][1]
+    if(!"tau.sq" %in% names(tuning)){stop("error: tau.sq must be specified in tuning value list")}
+    tau.sq.tuning <- tuning[["tau.sq"]][1]
   }
   
-  if(!"phi" %in% names(sp.tuning)){stop("error: phi must be specified in tuning value list")}
-  phi.tuning <- sp.tuning[["phi"]][1]
+  if(!"phi" %in% names(tuning)){stop("error: phi must be specified in tuning value list")}
+  phi.tuning <- tuning[["phi"]][1]
     
   if(cov.model == "matern"){
-    if(!"nu" %in% names(sp.tuning)){stop("error: nu must be specified in tuning value list")}
-    nu.tuning <- sp.tuning[["nu"]][1]
+    if(!"nu" %in% names(tuning)){stop("error: nu must be specified in tuning value list")}
+    nu.tuning <- tuning[["nu"]][1]
   }    
     
   storage.mode(phi.tuning) <- "double"
@@ -268,95 +317,127 @@ spLM <- function(formula, data = parent.frame(), coords, knots,
   storage.mode(tau.sq.tuning) <- "double"
   storage.mode(nu.tuning) <- "double"
   
+  ####################################################
+  ##Recover beta (removed for now)
+  ####################################################
+  n.samples <- batch.length*n.batch
+  
+  ## if(missing(end)){end <- n.samples}
+  
+  ## if(!is.numeric(start) || start >= n.samples)
+  ##   stop("error: invalid start for recovery of beta")
+  ## if(!is.numeric(end) || end > n.samples) 
+  ##   stop("error: invalid end for recovery of beta")
+  ## if(!is.numeric(thin) || thin >= n.samples) 
+  ##   stop("error: invalid thin for recovery of beta")
+
+  ##identifies which samples should be used for recovery
+  r.indx <- rep(0,n.samples)
+  ## r.indx[seq(start, end, by=as.integer(thin))] <- 1
+
+  ##switch off for now
+  get.beta <- FALSE
+  
+  storage.mode(r.indx) <- "integer"
+  storage.mode(get.beta) <- "integer"
   
   ####################################################
   ##Other stuff
   ####################################################
-  if(missing(n.samples)){stop("error: n.samples need to be specified")}
-
-  if(missing(sub.samples)){sub.samples <- c(1, n.samples, 1)}
-  if(length(sub.samples) != 3 || any(sub.samples > n.samples) ){stop("error: sub.samples misspecified")}
-  
-  storage.mode(n.samples) <- "integer"
   storage.mode(n.report) <- "integer"
   storage.mode(verbose) <- "integer"
 
+  ####################################################
+  ##Fix for no nugget pp model
+  ####################################################
+  if(is.pp && !nugget){
+    
+    if(!modified.pp){
+      modified.pp <- TRUE
+      warning("Because the specified predictive process model has no nugget, a modified predictive process was used.")
+    }
+
+    if(any(knots.coords.D == 0)){
+
+      jit <- 1000*.Machine$double.eps
+      warning(paste("Because the specified predictive process model has no nugget, a shift of distance ",jit," was added to those ", sum(knots.coords.D == 0), " knot locations that coincide with observed locations.",sep=""))
+      
+      while(any(knots.coords.D == 0)){
+        ind <- which(knots.coords.D == 0, arr.ind = TRUE)
+        knot.coords[ind[1,]] <- knot.coords[ind[1,]]+jit       
+        knots.coords.D <- iDist(knot.coords, coords)
+        knots.D <- iDist(knot.coords)
+      }
+    }
+     
+  }
 
   ####################################################
   ##Pack it up and off it goes
   ####################################################
-
-  ##force modified predictive process if nugget == 0
-  if(is.pp){
-    if(!nugget && !modified.pp){
-      warning("spLM: switching to modified predictive process for no nugget model.")
-      modified.pp <- TRUE
-      storage.mode(modified.pp) <- "integer"
-    }
-  }
-    
-
+  ptm <- proc.time()
   
   if(is.pp){
-    
-    out <- .Call("spPPLM", Y, X, p, n, coords.D,
-                 modified.pp, m, knots.D, coords.knots.D, nugget,               
-                 sigma.sq.IG, tau.sq.IG, nu.Unif, phi.Unif,
-                 phi.starting, sigma.sq.starting, tau.sq.starting, nu.starting, beta.starting,
-                 phi.tuning, sigma.sq.tuning, tau.sq.tuning, nu.tuning, 
-                 cov.model, n.samples, verbose, n.report)
+    out <- .Call("spPPLM", Y, X, p, n, m, coords.D, knots.D, knots.coords.D,
+                 modified.pp, beta.prior, beta.Norm, sigma.sq.IG, tau.sq.IG, nu.Unif, phi.Unif,
+                 beta.starting, phi.starting, sigma.sq.starting, tau.sq.starting, nu.starting,
+                 phi.tuning, sigma.sq.tuning, tau.sq.tuning, nu.tuning,
+                 nugget, cov.model, is.amcmc, n.batch, batch.length, accept.rate, verbose, n.report)
     
   }else{
-    
     out <- .Call("spLM", Y, X, p, n, coords.D,
-                 sigma.sq.IG, tau.sq.IG, nu.Unif, phi.Unif,
-                 phi.starting, sigma.sq.starting, tau.sq.starting, nu.starting, beta.starting,
-                 phi.tuning, sigma.sq.tuning, tau.sq.tuning, nu.tuning, 
-                 nugget, cov.model, n.samples, verbose, n.report)
-    
+                 beta.prior, beta.Norm, sigma.sq.IG, tau.sq.IG, nu.Unif, phi.Unif,
+                 phi.starting, sigma.sq.starting, tau.sq.starting, nu.starting,
+                 phi.tuning, sigma.sq.tuning, tau.sq.tuning, nu.tuning,
+                 get.beta, r.indx,
+                 nugget, cov.model, is.amcmc, n.batch, batch.length, accept.rate, verbose, n.report)   
   }
-
-  out$coords <- coords
-  out$is.pp <- is.pp
-  out$modified.pp <- modified.pp
   
-  if(is.pp){out$knot.coords <- knot.coords}
+  run.time <- proc.time() - ptm
+  
+  ##parameter names
+  if(is.pp){
+    out$p.beta.samples <- mcmc(t(out$p.beta.samples))
+    colnames(out$p.beta.samples) <- x.names
+  }## else{
+  ##   if(get.beta){
+  ##     out$p.beta.samples <- mcmc(t(out$p.beta.samples))
+  ##     colnames(out$p.beta.samples) <- x.names
+  ##   }
+  ## }
+  
+  out$p.theta.samples <- mcmc(t(out$p.theta.samples))
+  
+  col.names <- rep("null",ncol(out$p.theta.samples))
+  
+  if(!nugget && cov.model != "matern"){
+    col.names[1:2] <- c("sigma.sq", "phi")
+  }else if(nugget && cov.model != "matern"){
+    col.names[1:3] <- c("sigma.sq", "tau.sq", "phi")
+  }else if(!nugget && cov.model == "matern"){
+    col.names[1:3] <- c("sigma.sq", "phi", "nu")
+  }else{
+    col.names[1:4] <- c("sigma.sq", "tau.sq", "phi", "nu")
+  }
+  
+  colnames(out$p.theta.samples) <- col.names
   
   out$Y <- Y
   out$X <- X
-  out$n <- n
-  out$m <- m
-  out$p <- p
-  out$knots.D <- knots.D
-  out$coords.D <- coords.D
-  out$coords.knots.D <- coords.knots.D
+  out$coords <- coords
+  out$is.pp <- is.pp
+  if(is.pp){
+    out$knot.coords <- knot.coords
+    out$modified.pp <- modified.pp
+  }  
   out$cov.model <- cov.model
   out$nugget <- nugget
-  out$verbose <- verbose
-  #out$n.samples <- n.samples
-  out$sub.samples <- sub.samples
-  out$recovered.effects <- TRUE
-
-  ##subsample
-  out$sp.effects <- out$sp.effects[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]
-  if(is.pp){out$sp.effects.knots <- out$sp.effects.knots[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]}
-  out$p.samples <- mcmc(t(out$p.samples[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]))
-  out$n.samples <- nrow(out$p.samples)##get adjusted n.samples
-  
-  col.names <- rep("null",ncol(out$p.samples))
-  
-  col.names[1:p] <- x.names
-  if(!nugget && cov.model != "matern"){
-    col.names[(p+1):(p+2)] <- c("sigma.sq", "phi")
-  }else if(nugget && cov.model != "matern"){
-    col.names[(p+1):(p+3)] <- c("sigma.sq", "tau.sq", "phi")
-  }else if(!nugget && cov.model == "matern"){
-    col.names[(p+1):(p+3)] <- c("sigma.sq", "phi", "nu")
-  }else{
-    col.names[(p+1):(p+4)] <- c("sigma.sq", "tau.sq", "phi", "nu")
-  }
-    
-  colnames(out$p.samples) <- col.names
+  out$beta.prior <- beta.prior
+  out$beta.Norm <- beta.Norm
+  ## out$r.indx <- r.indx
+  ## out$get.beta <- get.beta
+  out$x.names <- x.names
+  out$run.time <- run.time
   
   class(out) <- "spLM"
   out  
