@@ -1,14 +1,3 @@
-// Andrew O. Finley
-// Dept. of Forest Resources
-// University of Minnesota
-// finleya@msu.edu 
-//
-// This software is distributed under the terms of the GNU GENERAL
-// PUBLIC LICENSE Version 2, June 1991.  See the package LICENSE
-// file for more information.
-//
-// Copyright (C) 2004 Andrew O. Finley
-
 #include <iostream>
 #include <iomanip>
 #include <string>
@@ -29,15 +18,21 @@ void updateThetaGibbs(double *x, double *y, int &nxrow, int &nxcol, double *fixe
 		      double *tmpXCol, double *tmpXCol1, string &thetaPrior, double *thetaPriorMu, double *thetaPriorV);
 
 void mvrnorm(double *des, double *mu, double * cholCov, int dim);
+
 void mvrnorm(double *des, double *mu, double * cholCov, int dim, bool upper);
 
-void showMatrix(double *x, int xnrow, int xncol);
-void writeRMatrix(string outfile, double * a, int nrow, int ncol);
+//void showMatrix(double *x, int xnrow, int xncol);
 
-SEXP getListElement (SEXP list, const char *str);
+//void writeRMatrix(string outfile, double * a, int nrow, int ncol);
+
+SEXP getList(SEXP list, const char *str);
+
+SEXP getGetList(SEXP list, const char *str1, const char *str2);
 
 void zeros(double *x, int length);
 
+void zeros(int *x, int length);
+
 void iden(double *x, int &nrow);
 
 void kron(double *a, int &dima1, int &dima2, 
@@ -52,14 +47,13 @@ double dTNorm(double x, double mu, double sd, double a, double b);
 
 void diagmm(int &nrow_b, int &ncol_b, double *a, double *b, double *c);
 
-
 void subsetCovRow(double *x, int n, int p, int begin, int end, double *cov, double *means);
+
 void subsetCovCol(double *x, int p, int begin, int end, double *cov, double *means);
 
-double mtrxInvLogDet(double *m, int dim, int info);
-
-void mtrxInv(double *m, int dim, int info);
+//double mtrxInvLogDet(double *m, int dim, int info);
 
+//void mtrxInv(double *m, int dim, int info);
 
 double logit(double theta, double a, double b);
 
@@ -81,8 +75,26 @@ double logit_logpost(int &n, double *Y, double *eta, double *w);
 
 double binomial_logpost(int &n, double *Y, double *eta, double *w, int *r);
 
-double poisson_logpost(int &n, double *Y, double *eta, double *w);
+double poisson_logpost(int &n, double *Y, double *eta, double *w, int *r);
+
+double binomial_logpost(int &n, double *Y, double *eta, int *r);
+
+double poisson_logpost(int &n, double *Y, double *eta, int *r);
 
 void report(int &s, int &nSamples, int &status, int &nReport, bool &verbose);
 
 /* void dnscsr(double *A, int nrowA, int ncolB, double *a, int *indxa, int *pntrb, int *pntre); */
+
+void spCor(double *D, int n, double *theta, string &covModel, double *C);
+
+double spCor(double D, double *theta, string &covModel);
+
+double spCor(double D, double phi, double nu, string &covModel);
+
+void spCov(double *D, int n, double *theta, string &covModel, double *C); 
+
+void spCovLT(double *D, int n, double *theta, string &covModel, double *C); 
+
+void transpose(double *m, int w, int h);
+
+void clearUT(double *m, int n);
